import { Provide } from '@midwayjs/decorator';
import { Config } from '@midwayjs/core';
import { FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import * as fs from 'node:fs';
import * as Path from 'node:path';
const moment = require('moment');
const sharp = require('sharp');
const sep = Path.sep;
const assert = require('assert');
@Provide()
export class FileService {
  @Config('export.root_path')
  root_path;

  async upload(files, fields, params) {
    const { appid, catalog, item } = params;
    assert(appid);
    const dirs = [appid];
    if (catalog && catalog !== '_') {
      const subs = catalog.split('_');
      dirs.push(...subs);
    }
    const rootPath = `${this.root_path}${sep}`;
    if (!rootPath) {
      throw new ServiceError(
        '服务端没有设置存储路径',
        FrameworkErrorEnum.SERVICE_FAULT
      );
    }
    if (!fs.existsSync(rootPath)) {
      // 如果不存在文件夹,就创建
      this.mkdir(rootPath);
    }
    // TODO: 检查分级目录是否存在，不存在则创建
    for (let i = 0; i < dirs.length; i++) {
      const p = `${rootPath}${sep}${dirs.slice(0, i + 1).join(sep)}`;
      if (!fs.existsSync(p)) {
        this.mkdir(p);
      }
    }
    // TODO: 指定文件名或者按时间生成文件名
    // const name = moment().format('YYYYMMDDHHmmss');
    const path = `${rootPath}${dirs.join(sep)}${sep}`;
    const filePath = `${path}`;
    for (const val of files) {
      let filename = item;
      if (!item) filename = moment().format('YYYYMMDDHHmmss');
      await sharp(val.data)
        .toFormat('jpg', { mozjpeg: true })
        .toFile(`${filePath}${filename}.jpg`);
      const uri = `/file/${dirs.join('/')}/${filename}.jpg`;
      return {
        id: filename,
        name: `${filename}.jpg`,
        uri,
      };
    }
  }
  // 创建文件夹
  mkdir(dirname) {
    if (fs.existsSync(dirname)) {
      return true;
    }
    if (this.mkdir(Path.dirname(dirname))) {
      fs.mkdirSync(dirname);
      return true;
    }
  }
}
